//
// Implements a very simple plugin that displays system call information in a
// readable form
//
// skape
// mmiller@hick.org
// 5/2007
//
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include "..\..\common\winstrace.h"
#include "ntapi.h"

#define DEFAULT_SYSCALL DefaultDisplaySystemCall

#define P_IN  0x1
#define P_OUT 0x2

typedef VOID (*DISPLAY_ROUTINE)(
		__in PSYSTEM_CALL_EVENT Event);

typedef VOID (*DISPLAY_ARGUMENT_ROUTINE)(
		__in PSYSTEM_CALL_EVENT Event,
		__in ULONG_PTR Parameter);

//
// Descriptor used to describe a system call that is to be displayed
//
typedef struct _SYSTEM_CALL_DISPLAY_DESCRIPTOR
{
	//
	// The hash code of the name of the string
	//
	ULONG           HashCode;

	//
	// The actual name of the system call
	//
	PWSTR           Name;

	//
	// The routine to use to display
	//
	DISPLAY_ROUTINE DisplayRoutine;

} SYSTEM_CALL_DISPLAY_DESCRIPTOR, *PSYSTEM_CALL_DISPLAY_DESCRIPTOR;

typedef enum _DISPLAY_STATE
{
	DISPLAY_CALL,
	DISPLAY_RETURN
} DISPLAY_STATE;

//
// The identifier of the thread that last had a system call issued
//
static ULONG LastEventThreadIdentifier = 0;

//
// The current state of the plugin
//
static DISPLAY_STATE DisplayState = DISPLAY_CALL;

////
//
// Utility routines
//
////

//
// Reads memory in the context of the process being traced
//
static BOOL ReadMemory(
		__in PSYSTEM_CALL_EVENT Event,
		__in PVOID Address,
		__out_bcount(BufferSize) PVOID Buffer,
		__in ULONG BufferSize)
{
	return ReadProcessMemory(
			Event->ProcessHandle,
			Address,
			Buffer,
			BufferSize,
			NULL);
}

////
//
// Common display routines
//
////

//
// Displays an invalid pointer
//
static VOID DisplayInvalidPointer(
		__in PVOID Address)
{
	wprintf(L"inv(%p)", Address);
}

//
// Displays a thread handle
//
static VOID DisplayThreadHandle(
		__in PSYSTEM_CALL_EVENT Event,
		__in ULONG_PTR ThreadHandle)
{
	if (ThreadHandle == (ULONG_PTR)-2)
		wprintf(L"NtCurrentThread()");
	else
		wprintf(L"%x", ThreadHandle);
}

//
// Displays a process handle
//
static VOID DisplayProcessHandle(
		__in PSYSTEM_CALL_EVENT Event,
		__in ULONG_PTR ProcessHandle)
{
	if (ProcessHandle == (ULONG_PTR)-1)
		wprintf(L"NtCurrentProcess()");
	else
		wprintf(L"%x", ProcessHandle);
}

//
// Displays an object attributes structure
//
static VOID DisplayObjectAttributes(
		__in PSYSTEM_CALL_EVENT Event,
		__in PVOID ObjectAttributesPtr)
{
	OBJECT_ATTRIBUTES Attributes;
	BOOL              Success = FALSE;

	do
	{
		//
		// Read the object attributes structure
		//
		if (!ReadMemory(
				Event,
				ObjectAttributesPtr,
				&Attributes,
				sizeof(Attributes)))
			break;

		wprintf(L"{Length=%lu,RootDirectory=%p,ObjectName=%p,Attributes=0x%x,SecurityDescriptor=%p,SecurityQoS=%p}",
				Attributes.Length,
				Attributes.RootDirectory,
				Attributes.ObjectName,
				Attributes.Attributes,
				Attributes.SecurityDescriptor,
				Attributes.SecurityQualityOfService);

		Success = TRUE;

	} while (0);

	if (!Success)
		DisplayInvalidPointer(
				ObjectAttributesPtr);
}

//
// Displays a value as a hex string
//
static VOID DisplayHexPointer(
		__in PSYSTEM_CALL_EVENT Event,
		__in ULONG_PTR ArgumentValue)
{
	wprintf(L"%p", ArgumentValue);
}

//
// Displays a value as a hex string
//
static VOID DisplayHexParameter(
		__in PSYSTEM_CALL_EVENT Event,
		__in ULONG_PTR ArgumentValue)
{
	wprintf(L"0x%x", ArgumentValue);
}

//
// Displays a system call by calling a function pointer for each parameter
//
static VOID DoDisplay(
		__in PSYSTEM_CALL_EVENT Event,
		...)
{
	ULONG_PTR Parameter;
	va_list   Ap;
	ULONG     Index = 0;

	wprintf(L"[%.8x] %s(",
			Event->ThreadId,
			Event->SystemCallName);

	va_start(Ap, Event);

	for (Index = 0;
		  Index < Event->NumberOfArguments;
		  Index++)
	{
		DISPLAY_ARGUMENT_ROUTINE ArgumentRoutine;
		PWSTR                    ArgumentName;
		ULONG                    ArgumentFlags;
		
		ArgumentName    = (PWSTR)va_arg(Ap, ULONG_PTR);
		ArgumentFlags   = (ULONG)va_arg(Ap, ULONG_PTR);
		ArgumentRoutine = (DISPLAY_ARGUMENT_ROUTINE)va_arg(Ap, ULONG_PTR);

		if (ArgumentName)
			wprintf(L"%s=", ArgumentName);

		if (ArgumentRoutine)
			ArgumentRoutine(
					Event,
					Event->Arguments[Index]);
		else
			DisplayHexParameter(
					Event,
					Event->Arguments[Index]);

		if (Index + 1 < Event->NumberOfArguments)
			wprintf(L",");
	}
	
	va_end(Ap);

	wprintf(L")");

}

//
// Default routine to display system calls that don't have a prototype
//
static VOID DefaultDisplaySystemCall(
		__in PSYSTEM_CALL_EVENT Event)
{
	ULONG Index;

	wprintf(L"[%.8x] %s(",
			Event->ThreadId,
			Event->SystemCallName);

	for (Index = 0;
		  Index < Event->NumberOfArguments;
		  Index++)
	{
		wprintf(L"0x%p", Event->Arguments[Index]);

		if (Index + 1 < Event->NumberOfArguments)
			wprintf(L",");
	}

	wprintf(L")");
}

////
//
// Specific display routines
//
////

static VOID OnNtQueryAttributesFile(__in PSYSTEM_CALL_EVENT Event)
{
	DoDisplay(Event, 
			L"ObjectAttributes", P_IN, DisplayObjectAttributes, 
			L"FileInformation",  P_OUT, DisplayHexPointer);
}

static VOID OnNtTerminateThread(__in PSYSTEM_CALL_EVENT Event)
{
	DoDisplay(Event, 
			L"ThreadHandle", P_IN, DisplayThreadHandle, 
			L"ExitCode",     P_IN, DisplayHexParameter);
}

//
// The lookup table for displaying system call information
//
SYSTEM_CALL_DISPLAY_DESCRIPTOR DisplayTable[] = 
{
	{ 1308328,     L"NtLockVirtualMemory",                  DEFAULT_SYSCALL },
	{ 3649840,     L"NtSetValueKey",                        DEFAULT_SYSCALL },
	{ 17850137,    L"NtSetSystemPowerState",                DEFAULT_SYSCALL },
	{ 20621400,    L"NtCompressKey",                        DEFAULT_SYSCALL },
	{ 20873553,    L"NtOpenChannel",                        DEFAULT_SYSCALL },
	{ 22638601,    L"NtSetInformationObject",               DEFAULT_SYSCALL },
	{ 22832657,    L"NtSetInformationThread",               DEFAULT_SYSCALL },
	{ 34468011,    L"NtFreeVirtualMemory",                  DEFAULT_SYSCALL },
	{ 40991515,    L"NtSavepointTransaction",               DEFAULT_SYSCALL },
	{ 51052781,    L"NtSetInformationFile",                 DEFAULT_SYSCALL },
	{ 53087617,    L"NtSavepointComplete",                  DEFAULT_SYSCALL },
	{ 55526375,    L"NtSetDefaultUILanguage",               DEFAULT_SYSCALL },
	{ 66729698,    L"NtSystemDebugControl",                 DEFAULT_SYSCALL },
	{ 74154485,    L"NtQueryInformationFile",               DEFAULT_SYSCALL },
	{ 84065450,    L"NtWaitForDebugEvent",                  DEFAULT_SYSCALL },
	{ 89560646,    L"NtOpenProcess",                        DEFAULT_SYSCALL },
	{ 104804811,   L"NtDeleteKey",                          DEFAULT_SYSCALL },
	{ 105459667,   L"NtUnloadKey",                          DEFAULT_SYSCALL },
	{ 118383900,   L"NtUnlockVirtualMemory",                DEFAULT_SYSCALL },
	{ 124747760,   L"NtAlpcQueryInformation",               DEFAULT_SYSCALL },
	{ 138098891,   L"NtRenameKey",                          DEFAULT_SYSCALL },
	{ 146422268,   L"NtOpenProcessTokenEx",                 DEFAULT_SYSCALL },
	{ 151004605,   L"NtCreateWaitablePort",                 DEFAULT_SYSCALL },
	{ 158746350,   L"NtMapUserPhysicalPages",               DEFAULT_SYSCALL },
	{ 165478610,   L"NtGetMUIRegistryInfo",                 DEFAULT_SYSCALL },
	{ 167864585,   L"NtSetInformationToken",                DEFAULT_SYSCALL },
	{ 172794672,   L"NtCompactKeys",                        DEFAULT_SYSCALL },
	{ 175214847,   L"NtQueryInformationPort",               DEFAULT_SYSCALL },
	{ 205882711,   L"NtOpenSection",                        DEFAULT_SYSCALL },
	{ 205886806,   L"NtOpenSession",                        DEFAULT_SYSCALL },
	{ 206273807,   L"NtPrePrepareEnlistment",               DEFAULT_SYSCALL },
	{ 217914811,   L"NtCreateMailslotFile",                 DEFAULT_SYSCALL },
	{ 223580747,   L"NtStopProfile",                        DEFAULT_SYSCALL },
	{ 232452796,   L"NtSetBootEntryOrder",                  DEFAULT_SYSCALL },
	{ 237806824,   L"NtRemoveIoCompletionEx",               DEFAULT_SYSCALL },
	{ 250833157,   L"NtSetDriverEntryOrder",                DEFAULT_SYSCALL },
	{ 258902512,   L"NtQueryInformationAtom",               DEFAULT_SYSCALL },
	{ 264973553,   L"NtAlpcDisconnectPort",                 DEFAULT_SYSCALL },
	{ 306605011,   L"NtOpenObjectAuditAlarm",               DEFAULT_SYSCALL },
	{ 324349184,   L"NtSetHighWaitLowThread",               DEFAULT_SYSCALL },
	{ 334175248,   L"NtSetTimerResolution",                 DEFAULT_SYSCALL },
	{ 335216605,   L"NtDebugActiveProcess",                 DEFAULT_SYSCALL },
	{ 357276952,   L"NtQueryTimerResolution",               DEFAULT_SYSCALL },
	{ 366760765,   L"NtQueryBootEntryOrder",                DEFAULT_SYSCALL },
	{ 372474689,   L"NtCancelTimer",                        DEFAULT_SYSCALL },
	{ 384628904,   L"NtSetSecurityObject",                  DEFAULT_SYSCALL },
	{ 390167896,   L"NtFilterToken",                        DEFAULT_SYSCALL },
	{ 394384230,   L"NtAlpcSendWaitReceivePort",            DEFAULT_SYSCALL },
	{ 407217459,   L"NtConnectPort",                        DEFAULT_SYSCALL },
	{ 424741095,   L"NtOpenEvent",                          DEFAULT_SYSCALL },
	{ 438854706,   L"NtGetNotificationResourceManager",     DEFAULT_SYSCALL },
	{ 440033346,   L"NtAccessCheck",                        DEFAULT_SYSCALL },
	{ 440956604,   L"NtAlpcOpenSenderThread",               DEFAULT_SYSCALL },
	{ 441210819,   L"NtIsUILanguageComitted",               DEFAULT_SYSCALL },
	{ 450635768,   L"NtRollbackEnlistment",                 DEFAULT_SYSCALL },
	{ 452092695,   L"NtQueryAttributesFile",                OnNtQueryAttributesFile },
	{ 455245913,   L"NtAlertThread",                        DEFAULT_SYSCALL },
	{ 485764581,   L"NtFreezeTransactions",                 DEFAULT_SYSCALL },
	{ 491154764,   L"NtCreateEvent",                        DEFAULT_SYSCALL },
	{ 494439423,   L"NtMapCMFModule",                       DEFAULT_SYSCALL },
	{ 500561148,   L"NtTerminateJobObject",                 DEFAULT_SYSCALL },
	{ 517413599,   L"NtCreateIoCompletion",                 DEFAULT_SYSCALL },
	{ 518936873,   L"NtQuerySecurityObject",                DEFAULT_SYSCALL },
	{ 521689094,   L"NtSetLowWaitHighThread",               DEFAULT_SYSCALL },
	{ 550054646,   L"NtReadOnlyEnlistment",                 DEFAULT_SYSCALL },
	{ 558372314,   L"NtOpenTimer",                          DEFAULT_SYSCALL },
	{ 559227321,   L"NtWriteFile",                          DEFAULT_SYSCALL },
	{ 566883859,   L"NtQueryOpenSubKeysEx",                 DEFAULT_SYSCALL },
	{ 579305679,   L"NtRequestWaitReplyPort",               DEFAULT_SYSCALL },
	{ 585434089,   L"NtQueryDefaultLocale",                 DEFAULT_SYSCALL },
	{ 590894546,   L"NtSetInformationResourceManager",      DEFAULT_SYSCALL },
	{ 591230533,   L"NtCreateToken",                        DEFAULT_SYSCALL },
	{ 606139417,   L"NtAccessCheckByTypeAndAuditAlarm",     DEFAULT_SYSCALL },
	{ 610702062,   L"NtProtectVirtualMemory",               DEFAULT_SYSCALL },
	{ 624785983,   L"NtCreateTimer",                        DEFAULT_SYSCALL },
	{ 642482131,   L"NtOpenPrivateNamespace",               DEFAULT_SYSCALL },
	{ 647587082,   L"NtQueryDebugFilterState",              DEFAULT_SYSCALL },
	{ 659269690,   L"NtRequestPort",                        DEFAULT_SYSCALL },
	{ 660277714,   L"NtTestAlert",                          DEFAULT_SYSCALL },
	{ 660359153,   L"NtQueryInformationWorkerFactory",      DEFAULT_SYSCALL },
	{ 680440366,   L"NtTraceControl",                       DEFAULT_SYSCALL },
	{ 689835989,   L"NtLoadKeyEx",                          DEFAULT_SYSCALL },
	{ 693124582,   L"NtRollforwardTransactionManager",      DEFAULT_SYSCALL },
	{ 706673878,   L"NtReplyWaitReplyPort",                 DEFAULT_SYSCALL },
	{ 708132656,   L"NtVdmControl",                         DEFAULT_SYSCALL },
	{ 721809116,   L"NtInitializeNlsFiles",                 DEFAULT_SYSCALL },
	{ 725202515,   L"NtQueryInformationResourceManager",    DEFAULT_SYSCALL },
	{ 727955738,   L"NtCreateJobSet",                       DEFAULT_SYSCALL },
	{ 738662378,   L"NtInitializeRegistry",                 DEFAULT_SYSCALL },
	{ 739852836,   L"NtReplaceKey",                         DEFAULT_SYSCALL },
	{ 740350423,   L"NtRemoveProcessDebug",                 DEFAULT_SYSCALL },
	{ 740694444,   L"NtSetEaFile",                          DEFAULT_SYSCALL },
	{ 741449051,   L"NtQueryMutant",                        DEFAULT_SYSCALL },
	{ 747294480,   L"NtEnumerateKey",                       DEFAULT_SYSCALL },
	{ 753405415,   L"NtRemoveIoCompletion",                 DEFAULT_SYSCALL },
	{ 756969469,   L"NtQuerySystemEnvironmentValueEx",      DEFAULT_SYSCALL },
	{ 775195421,   L"NtSetUuidSeed",                        DEFAULT_SYSCALL },
	{ 777207734,   L"NtReplyPort",                          DEFAULT_SYSCALL },
	{ 781105966,   L"NtStartProfile",                       DEFAULT_SYSCALL },
	{ 806911561,   L"NtUnloadKeyEx",                        DEFAULT_SYSCALL },
	{ 807963217,   L"NtQueryObject",                        DEFAULT_SYSCALL },
	{ 812936974,   L"NtCreateResourceManager",              DEFAULT_SYSCALL },
	{ 813151794,   L"NtResumeThread",                       DEFAULT_SYSCALL },
	{ 827924693,   L"NtClearAllSavepointsTransaction",      DEFAULT_SYSCALL },
	{ 828170613,   L"NtAllocateLocallyUniqueId",            DEFAULT_SYSCALL },
	{ 830291245,   L"NtAlpcCreatePortSection",              DEFAULT_SYSCALL },
	{ 839842776,   L"NtQueryDirectoryFile",                 DEFAULT_SYSCALL },
	{ 847820863,   L"NtQuerySection",                       DEFAULT_SYSCALL },
	{ 866041139,   L"NtSetInformationTransactionManager",   DEFAULT_SYSCALL },
	{ 874991911,   L"NtRestoreKey",                         DEFAULT_SYSCALL },
	{ 875002413,   L"NtQueryEaFile",                        DEFAULT_SYSCALL },
	{ 889135084,   L"NtQueryVirtualMemory",                 DEFAULT_SYSCALL },
	{ 904075004,   L"NtFlushVirtualMemory",                 DEFAULT_SYSCALL },
	{ 921771248,   L"NtWriteVirtualMemory",                 DEFAULT_SYSCALL },
	{ 930261999,   L"NtCancelIoFile",                       DEFAULT_SYSCALL },
	{ 947994144,   L"NtAlpcDeletePortSection",              DEFAULT_SYSCALL },
	{ 963624748,   L"NtCreateMutant",                       DEFAULT_SYSCALL },
	{ 979791898,   L"NtCreateDirectoryObject",              DEFAULT_SYSCALL },
	{ 981617155,   L"NtAlpcAcceptConnectPort",              DEFAULT_SYSCALL },
	{ 992224810,   L"NtClearEvent",                         DEFAULT_SYSCALL },
	{ 1023088693,  L"NtFlushProcessWriteBuffers",           DEFAULT_SYSCALL },
	{ 1026042918,  L"NtTraceEvent",                         DEFAULT_SYSCALL },
	{ 1030332970,  L"NtCreateThread",                       DEFAULT_SYSCALL },
	{ 1042224839,  L"NtSaveKeyEx",                          DEFAULT_SYSCALL },
	{ 1050443808,  L"NtGetTickCount",                       DEFAULT_SYSCALL },
	{ 1057351909,  L"NtWriteErrorLogEntry",                 DEFAULT_SYSCALL },
	{ 1059339514,  L"NtCreateFile",                         DEFAULT_SYSCALL },
	{ 1060446008,  L"NtResetEvent",                         DEFAULT_SYSCALL },
	{ 1116327450,  L"NtThawRegistry",                       DEFAULT_SYSCALL },
	{ 1126510593,  L"NtDeleteFile",                         DEFAULT_SYSCALL },
	{ 1130968816,  L"NtAreMappedFilesTheSame",              DEFAULT_SYSCALL },
	{ 1134642726,  L"NtAdjustPrivilegesToken",              DEFAULT_SYSCALL },
	{ 1141594664,  L"NtOpenMutant",                         DEFAULT_SYSCALL },
	{ 1160399876,  L"NtCreatePort",                         DEFAULT_SYSCALL },
	{ 1183945448,  L"NtQueryMultipleValueKey",              DEFAULT_SYSCALL },
	{ 1208302886,  L"NtOpenThread",                         DEFAULT_SYSCALL },
	{ 1217408024,  L"NtAlpcOpenSenderProcess",              DEFAULT_SYSCALL },
	{ 1218016543,  L"NtAddBootEntry",                       DEFAULT_SYSCALL },
	{ 1227883782,  L"NtUnloadKey2",                         DEFAULT_SYSCALL },
	{ 1231341600,  L"NtUnloadDriver",                       DEFAULT_SYSCALL },
	{ 1249067535,  L"NtAlpcCreateSectionView",              DEFAULT_SYSCALL },
	{ 1249433242,  L"NtQueryPerformanceCounter",            DEFAULT_SYSCALL },
	{ 1278816042,  L"NtQueryEvent",                         DEFAULT_SYSCALL },
	{ 1294217009,  L"NtPulseEvent",                         DEFAULT_SYSCALL },
	{ 1294286593,  L"NtUnlockFile",                         DEFAULT_SYSCALL },
	{ 1311258620,  L"NtDeleteAtom",                         DEFAULT_SYSCALL },
	{ 1324766233,  L"NtLoadDriver",                         DEFAULT_SYSCALL },
	{ 1366770434,  L"NtAlpcDeleteSectionView",              DEFAULT_SYSCALL },
	{ 1381664617,  L"NtWaitForMultipleObjects",             DEFAULT_SYSCALL },
	{ 1412447261,  L"NtQueryTimer",                         DEFAULT_SYSCALL },
	{ 1417006445,  L"NtSetThreadExecutionState",            DEFAULT_SYSCALL },
	{ 1418054439,  L"NtSetHighWaitLowEventPair",            DEFAULT_SYSCALL },
	{ 1454899145,  L"NtPlugPlayControl",                    DEFAULT_SYSCALL },
	{ 1468846673,  L"NtSetQuotaInformationFile",            DEFAULT_SYSCALL },
	{ 1478773778,  L"NtListenPort",                         DEFAULT_SYSCALL },
	{ 1504843158,  L"NtAlpcConnectPort",                    DEFAULT_SYSCALL },
	{ 1519488594,  L"NtSetLowWaitHighEventPair",            DEFAULT_SYSCALL },
	{ 1536235795,  L"NtFreeUserPhysicalPages",              DEFAULT_SYSCALL },
	{ 1551859967,  L"NtSetInformationProcess",              DEFAULT_SYSCALL },
	{ 1571751106,  L"NtLockRegistryKey",                    DEFAULT_SYSCALL },
	{ 1601843696,  L"NtCloseObjectAuditAlarm",              DEFAULT_SYSCALL },
	{ 1617396314,  L"NtRecoverResourceManager",             DEFAULT_SYSCALL },
	{ 1620570034,  L"NtReadFileScatter",                    DEFAULT_SYSCALL },
	{ 1622718769,  L"NtQueryValueKey",                      DEFAULT_SYSCALL },
	{ 1626329938,  L"NtSetVolumeInformationFile",           DEFAULT_SYSCALL },
	{ 1631050134,  L"NtQuerySymbolicLinkObject",            DEFAULT_SYSCALL },
	{ 1637475429,  L"NtAssignProcessToJobObject",           DEFAULT_SYSCALL },
	{ 1639769771,  L"NtNotifyChangeKey",                    DEFAULT_SYSCALL },
	{ 1656027476,  L"NtRenameTransactionManager",           DEFAULT_SYSCALL },
	{ 1666563199,  L"NtCancelSynchronousIoFile",            DEFAULT_SYSCALL },
	{ 1670795566,  L"NtShutdownWorkerFactory",              DEFAULT_SYSCALL },
	{ 1686167936,  L"NtQueryInformationProcess",            DEFAULT_SYSCALL },
	{ 1703644327,  L"NtIsSystemResumeAutomatic",            DEFAULT_SYSCALL },
	{ 1705215302,  L"NtCreateChannel",                      DEFAULT_SYSCALL },
	{ 1716438899,  L"NtSetInformationJobObject",            DEFAULT_SYSCALL },
	{ 1770563417,  L"NtAccessCheckAndAuditAlarm",           DEFAULT_SYSCALL },
	{ 1773902395,  L"NtCreateProcess",                      DEFAULT_SYSCALL },
	{ 1786933514,  L"NtQueryInformationToken",              DEFAULT_SYSCALL },
	{ 1791881336,  L"NtSetInformationEnlistment",           DEFAULT_SYSCALL },
	{ 1804620009,  L"NtQueryOleDirectoryFile",              DEFAULT_SYSCALL },
	{ 1806890642,  L"NtReadRequestData",                    DEFAULT_SYSCALL },
	{ 1837159242,  L"NtImpersonateClientOfPort",            DEFAULT_SYSCALL },
	{ 1840548926,  L"NtCreateProfile",                      DEFAULT_SYSCALL },
	{ 1841268270,  L"NtFsControlFile",                      DEFAULT_SYSCALL },
	{ 1842013552,  L"NtSetWinStationInformation",           DEFAULT_SYSCALL },
	{ 1851421308,  L"NtEnumerateDriverEntries",             DEFAULT_SYSCALL },
	{ 1853225831,  L"NtCreateSymbolicLinkObject",           DEFAULT_SYSCALL },
	{ 1869902086,  L"NtQueryDriverEntryOrder",              DEFAULT_SYSCALL },
	{ 1873541452,  L"NtCreateTransactionManager",           DEFAULT_SYSCALL },
	{ 1889724599,  L"NtOpenThreadToken",                    DEFAULT_SYSCALL },
	{ 1890224460,  L"NtCreateSection",                      DEFAULT_SYSCALL },
	{ 1905565516,  L"NtSetLdtEntries",                      DEFAULT_SYSCALL },
	{ 1941339974,  L"NtListenChannel",                      DEFAULT_SYSCALL },
	{ 1958539626,  L"NtWorkerFactoryWorkerReady",           DEFAULT_SYSCALL },
	{ 1973533856,  L"NtSetBootOptions",                     DEFAULT_SYSCALL },
	{ 1996635560,  L"NtQueryBootOptions",                   DEFAULT_SYSCALL },
	{ 2010096174,  L"NtResumeProcess",                      DEFAULT_SYSCALL },
	{ 2022438591,  L"NtDuplicateObject",                    DEFAULT_SYSCALL },
	{ 2022883482,  L"NtGetNextProcess",                     DEFAULT_SYSCALL },
	{ 2031195747,  L"NtOpenSymbolicLinkObject",             DEFAULT_SYSCALL },
	{ 2038863954,  L"NtSetDefaultHardErrorPort",            DEFAULT_SYSCALL },
	{ 2051511368,  L"NtOpenTransactionManager",             DEFAULT_SYSCALL },
	{ 2056601587,  L"NtFlushInstructionCache",              DEFAULT_SYSCALL },
	{ 2059041591,  L"NtOpenEventPair",                      DEFAULT_SYSCALL },
	{ 2068534334,  L"NtDeleteObjectAuditAlarm",             DEFAULT_SYSCALL },
	{ 2073391178,  L"NtOpenJobObject",                      DEFAULT_SYSCALL },
	{ 2088724733,  L"NtSetEventBoostPriority",              DEFAULT_SYSCALL },
	{ 2105364230,  L"NtAllocateVirtualMemory",              DEFAULT_SYSCALL },
	{ 2115182980,  L"NtQueryOpenSubKeys",                   DEFAULT_SYSCALL },
	{ 2115511899,  L"NtFlushBuffersFile",                   DEFAULT_SYSCALL },
	{ 2125455260,  L"NtCreateEventPair",                    DEFAULT_SYSCALL },
	{ 2139776126,  L"NtQuerySystemInformation",             DEFAULT_SYSCALL },
	{ 2139804847,  L"NtCreateJobObject",                    DEFAULT_SYSCALL },
	{ 2142143825,  L"NtExtendSection",                      DEFAULT_SYSCALL },
	{ 2159273669,  L"NtOpenTransaction",                    DEFAULT_SYSCALL },
	{ 2174065997,  L"NtReleaseMutant",                      DEFAULT_SYSCALL },
	{ 2176380355,  L"NtPullTransaction",                    DEFAULT_SYSCALL },
	{ 2187618161,  L"NtQueryInformationThread",             DEFAULT_SYSCALL },
	{ 2215308177,  L"NtMapViewOfSection",                   DEFAULT_SYSCALL },
	{ 2220311879,  L"NtQueryDefaultUILanguage",             DEFAULT_SYSCALL },
	{ 2257238390,  L"NtDeleteValueKey",                     DEFAULT_SYSCALL },
	{ 2259126093,  L"NtCompareTokens",                      DEFAULT_SYSCALL },
	{ 2261184148,  L"NtPrepareComplete",                    DEFAULT_SYSCALL },
	{ 2268649603,  L"NtQueryFullAttributesFile",            DEFAULT_SYSCALL },
	{ 2268815470,  L"NtReplyWaitReceivePortEx",             DEFAULT_SYSCALL },
	{ 2275181658,  L"NtWaitForMultipleObjects32",           DEFAULT_SYSCALL },
	{ 2279657854,  L"NtCancelDeviceWakeupRequest",          DEFAULT_SYSCALL },
	{ 2290740936,  L"NtTerminateThread",                    OnNtTerminateThread },
	{ 2305439312,  L"NtQueryInstallUILanguage",             DEFAULT_SYSCALL },
	{ 2310336135,  L"NtSetLowEventPair",                    DEFAULT_SYSCALL },
	{ 2315794566,  L"NtTerminateProcess",                   DEFAULT_SYSCALL },
	{ 2320379232,  L"NtFlushInstallUILanguage",             DEFAULT_SYSCALL },
	{ 2321937609,  L"NtWriteFileGather",                    DEFAULT_SYSCALL },
	{ 2337240375,  L"NtCreatePrivateNamespace",             DEFAULT_SYSCALL },
	{ 2342159702,  L"NtSuspendThread",                      DEFAULT_SYSCALL },
	{ 2349150070,  L"NtWriteRequestData",                   DEFAULT_SYSCALL },
	{ 2395833618,  L"NtDebugContinue",                      DEFAULT_SYSCALL },
	{ 2404411454,  L"NtDeletePrivateNamespace",             DEFAULT_SYSCALL },
	{ 2413705353,  L"NtFlushWriteBuffer",                   DEFAULT_SYSCALL },
	{ 2427877438,  L"NtOpenSemaphore",                      DEFAULT_SYSCALL },
	{ 2459470999,  L"NtAddDriverEntry",                     DEFAULT_SYSCALL },
	{ 2465182631,  L"NtAlpcCreateSecurityContext",          DEFAULT_SYSCALL },
	{ 2483218563,  L"NtSetHighEventPair",                   DEFAULT_SYSCALL },
	{ 2494291107,  L"NtCreateSemaphore",                    DEFAULT_SYSCALL },
	{ 2498694810,  L"NtCreateWinStation",                   DEFAULT_SYSCALL },
	{ 2500923302,  L"NtPowerInformation",                   DEFAULT_SYSCALL },
	{ 2546971579,  L"NtModifyBootEntry",                    DEFAULT_SYSCALL },
	{ 2547329240,  L"NtLoadKey",                            DEFAULT_SYSCALL },
	{ 2550600810,  L"NtWaitLowEventPair",                   DEFAULT_SYSCALL },
	{ 2561502531,  L"NtAllocateUuids",                      DEFAULT_SYSCALL },
	{ 2582885530,  L"NtAlpcDeleteSecurityContext",          DEFAULT_SYSCALL },
	{ 2592137659,  L"NtCreateProcessEx",                    DEFAULT_SYSCALL },
	{ 2594096777,  L"NtSuspendProcess",                     DEFAULT_SYSCALL },
	{ 2597727458,  L"NtOpenKey",                            DEFAULT_SYSCALL },
	{ 2625141072,  L"NtGetNextThread",                      DEFAULT_SYSCALL },
	{ 2625963423,  L"NtSetIoCompletion",                    DEFAULT_SYSCALL },
	{ 2628893623,  L"NtDeleteBootEntry",                    DEFAULT_SYSCALL },
	{ 2650016133,  L"NtGetNlsSectionPtr",                   DEFAULT_SYSCALL },
	{ 2663857625,  L"NtSaveKey",                            DEFAULT_SYSCALL },
	{ 2664712179,  L"NtAlpcQueryInformationMessage",        DEFAULT_SYSCALL },
	{ 2665754348,  L"NtWaitForWorkViaWorkerFactory",        DEFAULT_SYSCALL },
	{ 2676664726,  L"NtOpenWinStation",                     DEFAULT_SYSCALL },
	{ 2694487853,  L"NtSetSystemTime",                      DEFAULT_SYSCALL },
	{ 2715615880,  L"NtSetDefaultLocale",                   DEFAULT_SYSCALL },
	{ 2719075990,  L"NtGetPlugPlayEvent",                   DEFAULT_SYSCALL },
	{ 2725229929,  L"NtCreateThreadEx",                     DEFAULT_SYSCALL },
	{ 2730489880,  L"NtNotifyChangeMultipleKeys",           DEFAULT_SYSCALL },
	{ 2731668647,  L"NtCreateKeyedEvent",                   DEFAULT_SYSCALL },
	{ 2760937127,  L"NtDelayExecution",                     DEFAULT_SYSCALL },
	{ 2761654898,  L"NtIsProcessInJob",                     DEFAULT_SYSCALL },
	{ 2762378301,  L"NtDisplayString",                      DEFAULT_SYSCALL },
	{ 2785026404,  L"NtRollbackComplete",                   DEFAULT_SYSCALL },
	{ 2793381506,  L"NtSaveMergedKeys",                     DEFAULT_SYSCALL },
	{ 2799381649,  L"NtCreateEnlistment",                   DEFAULT_SYSCALL },
	{ 2817092279,  L"NtSignalAndWaitForSingleObject",       DEFAULT_SYSCALL },
	{ 2825631860,  L"NtAlpcCreatePort",                     DEFAULT_SYSCALL },
	{ 2828795822,  L"NtQuerySystemTime",                    DEFAULT_SYSCALL },
	{ 2866742117,  L"NtCreatePagingFile",                   DEFAULT_SYSCALL },
	{ 2878634288,  L"NtGetWriteWatch",                      DEFAULT_SYSCALL },
	{ 2883797381,  L"NtThawTransactions",                   DEFAULT_SYSCALL },
	{ 2889133938,  L"NtCancelIoFileEx",                     DEFAULT_SYSCALL },
	{ 2901753469,  L"NtListTransactions",                   DEFAULT_SYSCALL },
	{ 2909638563,  L"NtOpenKeyedEvent",                     DEFAULT_SYSCALL },
	{ 2917984673,  L"NtGetContextThread",                   DEFAULT_SYSCALL },
	{ 2917984685,  L"NtSetContextThread",                   DEFAULT_SYSCALL },
	{ 2934784941,  L"NtAlpcImpersonateClientOfPort",        DEFAULT_SYSCALL },
	{ 2944791460,  L"NtResetWriteWatch",                    DEFAULT_SYSCALL },
	{ 2945487520,  L"NtYieldExecution",                     DEFAULT_SYSCALL },
	{ 2968299646,  L"NtAlpcCreateResourceReserve",          DEFAULT_SYSCALL },
	{ 2977351565,  L"NtOpenEnlistment",                     DEFAULT_SYSCALL },
	{ 2993456022,  L"NtQueueApcThread",                     DEFAULT_SYSCALL },
	{ 3001235590,  L"NtCommitEnlistment",                   DEFAULT_SYSCALL },
	{ 3013261945,  L"NtFreezeRegistry",                     DEFAULT_SYSCALL },
	{ 3029108132,  L"NtRaiseException",                     DEFAULT_SYSCALL },
	{ 3030826643,  L"NtQueryInformationTransactionManager", DEFAULT_SYSCALL },
	{ 3050457531,  L"NtMapUserPhysicalPagesScatter",        DEFAULT_SYSCALL },
	{ 3050577845,  L"NtAddAtom",                            DEFAULT_SYSCALL },
	{ 3052667521,  L"NtReleaseSemaphore",                   DEFAULT_SYSCALL },
	{ 3052977287,  L"NtClearSavepointTransaction",          DEFAULT_SYSCALL },
	{ 3062806891,  L"NtCommitComplete",                     DEFAULT_SYSCALL },
	{ 3078786437,  L"NtPrivilegeCheck",                     DEFAULT_SYSCALL },
	{ 3081885563,  L"NtSetInformationDebugObject",          DEFAULT_SYSCALL },
	{ 3086002545,  L"NtAlpcDeleteResourceReserve",          DEFAULT_SYSCALL },
	{ 3087915602,  L"NtQueryQuotaInformationFile",          DEFAULT_SYSCALL },
	{ 3089719470,  L"NtFlushKey",                           DEFAULT_SYSCALL },
	{ 3097488567,  L"NtW32Call",                            DEFAULT_SYSCALL },
	{ 3102710907,  L"NtNotifyChangeDirectoryFile",          DEFAULT_SYSCALL },
	{ 3117622650,  L"NtOpenIoCompletion",                   DEFAULT_SYSCALL },
	{ 3145916574,  L"NtEnumerateSystemEnvironmentValuesEx", DEFAULT_SYSCALL },
	{ 3151752287,  L"NtRecoverTransactionManager",          DEFAULT_SYSCALL },
	{ 3186075756,  L"NtClose",                              DEFAULT_SYSCALL },
	{ 3189731833,  L"NtApphelpCacheControl",                DEFAULT_SYSCALL },
	{ 3202656354,  L"NtPrivilegeObjectAuditAlarm",          DEFAULT_SYSCALL },
	{ 3204130009,  L"NtRegisterThreadTerminatePort",        DEFAULT_SYSCALL },
	{ 3216438668,  L"NtGetCurrentProcessorNumber",          DEFAULT_SYSCALL },
	{ 3229129615,  L"NtDuplicateToken",                     DEFAULT_SYSCALL },
	{ 3239665610,  L"NtQueryKey",                           DEFAULT_SYSCALL },
	{ 3244988050,  L"NtCallbackReturn",                     DEFAULT_SYSCALL },
	{ 3252813291,  L"NtPrivilegedServiceAuditAlarm",        DEFAULT_SYSCALL },
	{ 3254519924,  L"NtOpenProcessToken",                   DEFAULT_SYSCALL },
	{ 3260253714,  L"NtReplyWaitSendChannel",               DEFAULT_SYSCALL },
	{ 3281952385,  L"NtQuerySemaphore",                     DEFAULT_SYSCALL },
	{ 3303927991,  L"NtCreateDebugObject",                  DEFAULT_SYSCALL },
	{ 3323485448,  L"NtSetDebugFilterState",                DEFAULT_SYSCALL },
	{ 3335507828,  L"NtQueryInformationJobObject",          DEFAULT_SYSCALL },
	{ 3336257519,  L"NtSetInformationWorkerFactory",        DEFAULT_SYSCALL },
	{ 3340127751,  L"NtDeviceIoControlFile",                DEFAULT_SYSCALL },
	{ 3351846118,  L"NtStartTm",                            DEFAULT_SYSCALL },
	{ 3357181704,  L"NtCompleteConnectPort",                DEFAULT_SYSCALL },
	{ 3363290549,  L"NtRegisterProtocolAddressInformation", DEFAULT_SYSCALL },
	{ 3391031446,  L"NtAcceptConnectPort",                  DEFAULT_SYSCALL },
	{ 3392085129,  L"NtSecureConnectPort",                  DEFAULT_SYSCALL },
	{ 3408997833,  L"NtSetEvent",                           DEFAULT_SYSCALL },
	{ 3413163694,  L"NtShutdownSystem",                     DEFAULT_SYSCALL },
	{ 3423562520,  L"NtOpenResourceManager",                DEFAULT_SYSCALL },
	{ 3431686539,  L"NtRaiseHardError",                     DEFAULT_SYSCALL },
	{ 3432867835,  L"NtSetSystemEnvironmentValueEx",        DEFAULT_SYSCALL },
	{ 3454318713,  L"NtLockProductActivationKeys",          DEFAULT_SYSCALL },
	{ 3456426183,  L"NtAccessCheckByTypeResultListAndAuditAlarmByHandle", DEFAULT_SYSCALL },
	{ 3474734488,  L"NtReadFile",                           DEFAULT_SYSCALL },
	{ 3509902285,  L"NtModifyDriverEntry",                  DEFAULT_SYSCALL },
	{ 3522324498,  L"NtWaitForSingleObject",                DEFAULT_SYSCALL },
	{ 3525836444,  L"NtEnumerateValueKey",                  DEFAULT_SYSCALL },
	{ 3530075382,  L"NtEnumerateBootEntries",               DEFAULT_SYSCALL },
	{ 3542629052,  L"NtSetTimer",                           DEFAULT_SYSCALL },
	{ 3544273615,  L"NtRollbackSavepointTransaction",       DEFAULT_SYSCALL },
	{ 3576236695,  L"NtLoadKey2",                           DEFAULT_SYSCALL },
	{ 3590417444,  L"NtOpenDirectoryObject",                DEFAULT_SYSCALL },
	{ 3607132014,  L"NtAllocateUserPhysicalPages",          DEFAULT_SYSCALL },
	{ 3619760052,  L"NtAccessCheckByTypeResultList",        DEFAULT_SYSCALL },
	{ 3621572990,  L"NtSetInformationTransaction",          DEFAULT_SYSCALL },
	{ 3623484076,  L"NtSinglePhaseReject",                  DEFAULT_SYSCALL },
	{ 3637596402,  L"NtQueryPortInformationProcess",        DEFAULT_SYSCALL },
	{ 3639109663,  L"NtMakePermanentObject",                DEFAULT_SYSCALL },
	{ 3641922567,  L"NtGetDevicePowerState",                DEFAULT_SYSCALL },
	{ 3642639506,  L"NtLockFile",                           DEFAULT_SYSCALL },
	{ 3643859119,  L"NtDeleteDriverEntry",                  DEFAULT_SYSCALL },
	{ 3656710286,  L"NtAccessCheckByType",                  DEFAULT_SYSCALL },
	{ 3659548565,  L"NtOpenFile",                           DEFAULT_SYSCALL },
	{ 3691496109,  L"NtAlpcCancelMessage",                  DEFAULT_SYSCALL },
	{ 3694454461,  L"NtAdjustGroupsToken",                  DEFAULT_SYSCALL },
	{ 3694656117,  L"NtTranslateFilePath",                  DEFAULT_SYSCALL },
	{ 3727443335,  L"NtFindAtom",                           DEFAULT_SYSCALL },
	{ 3727838636,  L"NtContinue",                           DEFAULT_SYSCALL },
	{ 3731130084,  L"NtSendWaitReplyChannel",               DEFAULT_SYSCALL },
	{ 3737576145,  L"NtReleaseWorkerFactoryWorker",         DEFAULT_SYSCALL },
	{ 3740300583,  L"NtInitiatePowerAction",                DEFAULT_SYSCALL },
	{ 3741139149,  L"NtClearMUILicenseInfo",                DEFAULT_SYSCALL },
	{ 3755880959,  L"NtQueryInformationTransaction",        DEFAULT_SYSCALL },
	{ 3759623467,  L"NtCreateWorkerFactory",                DEFAULT_SYSCALL },
	{ 3791115442,  L"NtQueryVolumeInformationFile",         DEFAULT_SYSCALL },
	{ 3806051235,  L"NtImpersonateAnonymousToken",          DEFAULT_SYSCALL },
	{ 3826251678,  L"NtAlertResumeThread",                  DEFAULT_SYSCALL },
	{ 3843615418,  L"NtCreateTransaction",                  DEFAULT_SYSCALL },
	{ 3860091182,  L"NtRollbackTransaction",                DEFAULT_SYSCALL },
	{ 3861433233,  L"NtWaitHighEventPair",                  DEFAULT_SYSCALL },
	{ 3875954742,  L"NtMarshallTransaction",                DEFAULT_SYSCALL },
	{ 3893102782,  L"NtPrepareEnlistment",                  DEFAULT_SYSCALL },
	{ 3894177532,  L"NtRequestDeviceWakeup",                DEFAULT_SYSCALL },
	{ 3899557621,  L"NtReplyWaitReceivePort",               DEFAULT_SYSCALL },
	{ 3907717302,  L"NtSetContextChannel",                  DEFAULT_SYSCALL },
	{ 3908888738,  L"NtOpenThreadTokenEx",                  DEFAULT_SYSCALL },
	{ 3927379962,  L"NtSetIntervalProfile",                 DEFAULT_SYSCALL },
	{ 3942109896,  L"NtReleaseKeyedEvent",                  DEFAULT_SYSCALL },
	{ 3950481666,  L"NtQueryIntervalProfile",               DEFAULT_SYSCALL },
	{ 3956666840,  L"NtQueryInformationEnlistment",         DEFAULT_SYSCALL },
	{ 3959739485,  L"NtGetMUILicenseInfo",                  DEFAULT_SYSCALL },
	{ 3978616774,  L"NtCommitTransaction",                  DEFAULT_SYSCALL },
	{ 4006799056,  L"NtQueryWinStationInformation",         DEFAULT_SYSCALL },
	{ 4025245478,  L"NtMakeTemporaryObject",                DEFAULT_SYSCALL },
	{ 4026256471,  L"NtSetSystemEnvironmentValue",          DEFAULT_SYSCALL },
	{ 4029375240,  L"NtPropagationComplete",                DEFAULT_SYSCALL },
	{ 4031781398,  L"NtAccessCheckByTypeResultListAndAuditAlarm", DEFAULT_SYSCALL },
	{ 4060026779,  L"NtSetInformationKey",                  DEFAULT_SYSCALL },
	{ 4077592484,  L"NtRecoverEnlistment",                  DEFAULT_SYSCALL },
	{ 4110859456,  L"NtWaitForKeyedEvent",                  DEFAULT_SYSCALL },
	{ 4160564440,  L"NtQuerySystemEnvironmentValue",        DEFAULT_SYSCALL },
	{ 4161922559,  L"NtUnmapViewOfSection",                 DEFAULT_SYSCALL },
	{ 4162465997,  L"NtCreateNamedPipeFile",                DEFAULT_SYSCALL },
	{ 4177390023,  L"NtImpersonateThread",                  DEFAULT_SYSCALL },
	{ 4194623666,  L"NtPropagationFailed",                  DEFAULT_SYSCALL },
	{ 4212593363,  L"NtPrePrepareComplete",                 DEFAULT_SYSCALL },
	{ 4232355596,  L"NtQueryDirectoryObject",               DEFAULT_SYSCALL },
	{ 4242540566,  L"NtAlpcSetInformation",                 DEFAULT_SYSCALL },
	{ 4245032352,  L"NtQueryIoCompletion",                  DEFAULT_SYSCALL },
	{ 4247055249,  L"NtQueryLicenseValue",                  DEFAULT_SYSCALL },
	{ 4262263965,  L"NtReadVirtualMemory",                  DEFAULT_SYSCALL },
	{ 4269957917,  L"NtSetSystemInformation",               DEFAULT_SYSCALL },
	{ 4282069207,  L"NtCreateKey",                          DEFAULT_SYSCALL },
	{ 4284392430,  L"NtRequestWakeupLatency",               DEFAULT_SYSCALL },
};

PSYSTEM_CALL_DISPLAY_DESCRIPTOR LookupSystemCall(
		__in ULONG HashCode,
		__in PWSTR SystemCallName)
{
	PSYSTEM_CALL_DISPLAY_DESCRIPTOR CurrentDescriptor = NULL;
	ULONG                           CurrentIndex;
	ULONG                           LeftIndex = 0;
	ULONG                           RightIndex = ARRAYSIZE(DisplayTable) - 1;

	//
	// Find the descriptor associated with it in the binary sorted array
	//
	while (LeftIndex <= RightIndex)
	{
		CurrentIndex      = (LeftIndex + RightIndex) / 2;
		CurrentDescriptor = &DisplayTable[CurrentIndex];

		if (HashCode == CurrentDescriptor->HashCode)
			break;
		else if (HashCode > CurrentDescriptor->HashCode)
			LeftIndex = CurrentIndex + 1;
		else if (HashCode < CurrentDescriptor->HashCode)
			RightIndex = CurrentIndex - 1;
	}

	//
	// Did we find a matching name?
	//
	if (wcscmp(CurrentDescriptor->Name, SystemCallName))
		CurrentDescriptor = NULL;

	return CurrentDescriptor;
}

//
// Notified by the engine when a system call occurs
//
VOID OnSystemCall(
		__in PSYSTEM_CALL_EVENT Event)
{
	PSYSTEM_CALL_DISPLAY_DESCRIPTOR CurrentDescriptor;

	if (DisplayState == DISPLAY_RETURN)
		wprintf(L"\n");

	//
	// Save the unique identifier of the thread that last caused the event.  This
	// is used to control how the return value of the system call is displayed.
	//
	LastEventThreadIdentifier = Event->ThreadId;

	//
	// Find the display descriptor associated with this system call
	//
	CurrentDescriptor = LookupSystemCall(
			Event->SystemCallHash,
			Event->SystemCallName);

	if (!CurrentDescriptor)
		DefaultDisplaySystemCall(Event);
	else
		CurrentDescriptor->DisplayRoutine(
				Event);

	DisplayState = DISPLAY_RETURN;
}

//
// Notified by the engine when a system call return occurs
//
VOID OnSystemCallReturn(
		__in PSYSTEM_CALL_RETURN_EVENT Event)
{
	if (Event->SystemCall->ThreadId == LastEventThreadIdentifier)
	{
	   if (DisplayState == DISPLAY_RETURN)
			wprintf(L" = 0x%.8x\n", Event->Status);
	}
	else
	{
		if (DisplayState == DISPLAY_RETURN)
			wprintf(L"\n");

		wprintf(L"[%.8x] ... %s() = 0x%.8x\n", 
				Event->SystemCall->ThreadId,
				Event->SystemCall->SystemCallName,
				Event->Status);
	}

	DisplayState = DISPLAY_CALL;
}

//
// Nothin
//
PVOID InitializePlugin()
{
	return NULL;
}
