//
// Dumps information about IOCTLs invoked while system calls are being monitored
// in a given process
//
// skape
// mmiller@hick.org
// 6/2007
//
#include <windows.h>
#include <winioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "..\..\common\winstrace.h"

//
// Pre-computed hash codes
//
#define NtDeviceIoControlFileHash 3340127751
#define NtCreateFileHash          1059339514
#define NtCloseHash               3186075756

typedef union _CTL_CODE
{
	struct
	{
		ULONG Method     : 2;
		ULONG Function   : 12;
		ULONG Access     : 2;
		ULONG DeviceType : 16;
	};
	ULONG Ul;
} CTL_CODE, *PCTL_CODE;

typedef struct _IOCTL_PARAMETERS
{
	HANDLE    Device;
	HANDLE    Event;
	ULONG_PTR ApcRoutine;
	ULONG_PTR ApcContext;
	ULONG_PTR IoStatusBlock;
	CTL_CODE  IoControlCode;
	PVOID     InputBuffer;
	ULONG     InputBufferLength;
	PVOID     OutputBuffer;
	ULONG     OutputBufferLength;
} IOCTL_PARAMETERS, *PIOCTL_PARAMETERS;

typedef struct _UNICODE_STRING 
{
	USHORT Length;
	USHORT MaximumLength;
	PWSTR  Buffer;
} UNICODE_STRING, *PUNICODE_STRING;

LONG NtQueryObject(
		__in HANDLE ObjectHandle,
		__in ULONG InformationClass,
		__out PVOID Buffer,
		__in ULONG BufferLength,
		__out PULONG ReturnLength);

//
// Being lazy here, but it beats having to dynamically allocate or something.
//
#define DEVICE_NAME_BUFFER_SIZE sizeof(DeviceNameBuffer)
static CHAR            DeviceNameBuffer[65540];
static PUNICODE_STRING DeviceName = (PUNICODE_STRING)DeviceNameBuffer;

//
// Converts a control code's access type to a readable string
//
static PWSTR CtlCodeAccessToString(
		__in ULONG Access)
{
	if (Access == 0)
		return L"any";
	else if (Access == 1)
		return L"read";
	else if (Access == 2)
		return L"write";

	return L"unk";
}

//
// Converts a control code's buffering method to a readable string
//
static PWSTR CtlCodeMethodToString(
		__in ULONG Method)
{
	if (Method == 0)
		return L"buffered";
	else if (Method == 1)
		return L"indirect";
	else if (Method == 2)
		return L"outdirect";
	else if (Method == 3)
		return L"neither";

	return L"unk";
}

//
// Displays the supplied buffer
//
static VOID ShowBuffer(
		__in PBYTE Buffer,
		__in ULONG BufferLength)
{
	ULONG Offset = 0;
	ULONG RowIndex;

	while (Offset < BufferLength)
	{
		wprintf(L"%.8x:", Offset);

		//
		// Show hex bytes
		//
		for (RowIndex = 0; 
		     RowIndex < 16 && Offset + RowIndex < BufferLength; 
		     RowIndex++)
			wprintf(L" %02x", Buffer[Offset + RowIndex] & 0xff);

		//
		// Padding needed?
		//
		if (BufferLength - Offset < 16)
		{
			for (RowIndex = 0; 
			     RowIndex < 16 - (BufferLength - Offset);
			     RowIndex++)
				wprintf(L"   ");
		}

		wprintf(L"  ");

		//
		// Show printable bytes
		//
		for (RowIndex = 0; 
		     RowIndex < 16 && Offset + RowIndex < BufferLength; 
		     RowIndex++)
			wprintf(L"%C", 
					isprint((UCHAR)(Buffer[Offset + RowIndex])) ? (UCHAR)Buffer[Offset + RowIndex] : L'.');

		wprintf(L"\n");

		Offset += 16;
	}
}

//
// Displays the contents of a buffer from the remote process
//
static VOID DumpBuffer(
		__in PSYSTEM_CALL_RETURN_EVENT Event,
		__in PWSTR BufferType,
		__in PVOID RemoteBuffer,
		__in ULONG RemoteBufferLength)
{
	PBYTE LocalBuffer = NULL;

	do
	{
		//
		// Allocate us some storage
		//
		LocalBuffer = (PBYTE)malloc(RemoteBufferLength);

		if (!LocalBuffer)
		{
			wprintf(L"Failed to allocate dumpioctl buffer (%lu).\n", 
					RemoteBufferLength);
			break;
		}

		ZeroMemory(
				LocalBuffer,
				RemoteBufferLength);

		//
		// Read the contents of the buffer from the target process
		//
		if (!ReadProcessMemory(
				Event->SystemCall->ProcessHandle,
				RemoteBuffer,
				LocalBuffer,
				RemoteBufferLength,
				NULL))
		{
			wprintf(L"ReadProcessMemory(%p) failed, %lu.\n",
					RemoteBuffer,
					GetLastError());
			break;
		}

		//
		// Display the contents of the buffer
		//
		ShowBuffer(
				LocalBuffer,
				RemoteBufferLength);

	} while (0);

	//
	// Free it on up
	//
	if (LocalBuffer)
		free(LocalBuffer);
}

//
// Gets the object name associated with a handle from a remote process.
//
static PUNICODE_STRING LookupHandleObjectName(
		__in HANDLE ProcessHandle,
		__in HANDLE DeviceHandle)
{
	HANDLE LocalDeviceHandle = NULL;
	BOOL   Success = FALSE;

	do
	{
		//
		// Dupliate the handle into our process so that we can query it
		//
		if (!DuplicateHandle(
				ProcessHandle,
				DeviceHandle,
				GetCurrentProcess(),
				&LocalDeviceHandle,
				0,
				FALSE,
				DUPLICATE_SAME_ACCESS))
		{
			wprintf(L"DuplicateHandle failed, %lu.\n",
					GetLastError());
			break;
		}

		//
		// Query the name of the object using the ObjectNameInformation info
		// class
		//
		if (NtQueryObject(
				LocalDeviceHandle,
				1,
				DeviceName,
				DEVICE_NAME_BUFFER_SIZE,
				NULL) != 0)
		{
			wprintf(L"NtQueryObject failed.\n");
			break;
		}

		Success = TRUE;

	} while (0);

	if (LocalDeviceHandle)
		CloseHandle(
				LocalDeviceHandle);

	//
	// If we failed, then we'll use the handle
	//
	if (!Success)
	{
		swprintf(
				(PWSTR)(DeviceNameBuffer + 4),
				L"%x",
				DeviceHandle);

		DeviceName->Length        = wcslen((PWSTR)DeviceNameBuffer) * 2;
		DeviceName->MaximumLength = (wcslen((PWSTR)DeviceNameBuffer) + 1) * 2;
		DeviceName->Buffer        = (PWSTR)(DeviceNameBuffer + 4);
	}

	return DeviceName;
}

//
// Notified by the engine when a system call return occurs
//
VOID OnSystemCallReturn(
		__in PSYSTEM_CALL_RETURN_EVENT Event)
{
	PIOCTL_PARAMETERS Parameters = (PIOCTL_PARAMETERS)Event->SystemCall->Arguments;
	PUNICODE_STRING   DeviceName = NULL;

	//
	// These are not the ioctls you are looking for...
	//
	if (Event->SystemCall->SystemCallHash != NtDeviceIoControlFileHash)
		return;

	//
	// Get the name of the object associated with the device handle
	//
	DeviceName = LookupHandleObjectName(
			Event->SystemCall->ProcessHandle,
			Parameters->Device);

	//
	// Dump information about the IOCTL
	//
	wprintf(L"[hdev=%wZ,ctlcode=0x%x(access=%s,method=%s,func=%x),inlen=%lu,outlen=%lu,status=%.8x]\n",
			DeviceName,
			Parameters->IoControlCode.Ul,
			CtlCodeAccessToString(Parameters->IoControlCode.Access),
			CtlCodeMethodToString(Parameters->IoControlCode.Method),
			Parameters->IoControlCode.Function,
			Parameters->InputBufferLength,
			Parameters->OutputBufferLength,
			Event->Status);

	if (Parameters->InputBufferLength)
		DumpBuffer(
				Event, 
				L"Input",
				Parameters->InputBuffer,
				Parameters->InputBufferLength);
}

//
// Nothin
//
PVOID InitializePlugin()
{
	return NULL;
}
